@@ -1,7 +1,9 @@
 class CompUnit::Repository::Staging is CompUnit::Repository::Installation {
     has Str $.name;
 
-    submethod BUILD(Str :$!name --> Nil) { }
+    submethod BUILD(Str :$!name --> Nil) {
+        CompUnit::RepositoryRegistry.register-name($!name, self);
+    }
 
     method short-id() { 'staging' }
 
@@ -9,10 +11,10 @@ class CompUnit::Repository::Staging is CompUnit::Repository::Installation {
         $!name
     }
     method path-spec(CompUnit::Repository::Staging:D:) {
-        self.^name ~ '#name(' ~ $!name ~ ')#' ~ $.prefix.abspath;
+        self.^name ~ '#name(' ~ $!name ~ ')#' ~ $.prefix.absolute;
     }
     method source-file(Str $name --> IO::Path) {
-        my $file = self.prefix.child($name);
+        my $file = self.prefix.add($name);
         $file.e ?? $file !! self.next-repo.source-file($name)
     }
 }
