@@ -1,5 +1,6 @@
 my class Pair                   { ... }
 my class Range                  { ... }
+my class Seq                    { ... }
 my class X::Adverb              { ... }
 my class X::Bind                { ... }
 my class X::Bind::Slice         { ... }
@@ -58,6 +59,9 @@ my class Any { # declared in BOOTSTRAP
     proto method eager(|) is nodal { * }
     multi method eager() { self.list.eager }
 
+    proto method serial(|) is nodal { * }
+    multi method serial() { self }
+
     # derived from .list
     proto method List(|) is nodal { * }
     multi method List() { self.list }
@@ -66,7 +70,7 @@ my class Any { # declared in BOOTSTRAP
     proto method Array(|) is nodal { * }
     multi method Array() { self.list.Array }
     proto method Seq(|) is nodal { * }
-    multi method Seq() { self.list.Seq }
+    multi method Seq() { Seq.new(self.iterator) }
 
     proto method hash(|) is nodal { * }
     multi method hash(Any:U:) { my % = () }
@@ -403,10 +407,14 @@ my class Any { # declared in BOOTSTRAP
           my $scalar,
           Scalar,
           '$!whence',
+          # NOTE: even though the signature indicates a non-concrete SELF,
+          # by the time the below code is executed, it *may* have become
+          # concrete: and then we don't want the execution to reset it to
+          # an empty Hash.
           -> { nqp::if(
                  nqp::isconcrete(SELF),
                  SELF,
-                 (SELF = Hash.new)
+                 (SELF = nqp::create(Hash))
                ).BIND-KEY(key, $scalar)
              }
         )
@@ -479,7 +487,7 @@ proto sub infix:<===>(Mu $?, Mu $?) is pure { * }
 multi sub infix:<===>($?)    { Bool::True }
 multi sub infix:<===>(\a, \b) {
     nqp::p6bool(
-      nqp::eqaddr(a,b)
+      nqp::eqaddr(nqp::decont(a),nqp::decont(b))
       || (nqp::eqaddr(a.WHAT,b.WHAT)
            && nqp::iseq_s(nqp::unbox_s(a.WHICH), nqp::unbox_s(b.WHICH)))
     )
@@ -604,7 +612,9 @@ sub dd(|) {
         }
     }
     else { # tell where we are
-        note .name ?? "{lc .^name} {.name}" !! "({lc .^name})"
+        note .name
+          ?? "{lc .^name} {.name}{.signature.gist}"
+          !! "{lc .^name} {.signature.gist}"
           with callframe(1).code;
     }
     return
