@@ -102,6 +102,7 @@ my role Blob[::T = uint8] does Positional[T] does Stringy is repr('VMArray') is
     }
 
     multi method Bool(Blob:D:) { nqp::p6bool(nqp::elems(self)) }
+    method Capture(Blob:D:) { self.List.Capture }
 
     multi method elems(Blob:D:)   { nqp::p6box_i(nqp::elems(self)) }
     multi method elems(Blob:U: --> 1)   { }
@@ -136,7 +137,14 @@ my role Blob[::T = uint8] does Positional[T] does Stringy is repr('VMArray') is
     }
 
     multi method gist(Blob:D:) {
-        self.^name ~ ':0x<' ~ self.list.fmt('%02x', ' ') ~ '>'
+        self.^name ~ ':0x<' ~ self.map( -> \el {
+            state $i = 0;
+            ++$i == 101 ?? '...'
+                !! $i == 102 ?? last()
+                    !! nqp::if(nqp::iseq_i( # el.fmt: '%02x'
+                        nqp::chars(my str $v = nqp::lc(el.base: 16)),1),
+                        nqp::concat('0',$v),$v)
+        }) ~ '>'
     }
     multi method perl(Blob:D:) {
         self.^name ~ '.new(' ~ self.join(',') ~ ')';
@@ -592,6 +600,17 @@ my role Buf[::T = uint8] does Blob[T] is repr('VMArray') is array_type(T) {
             ?? self!push-list($action,self,@values)
             !! self!unshift-list($action,self,@values)
     }
+
+    method subbuf-rw($from = 0, $elems = self.elems - $from) is rw {
+        my Blob $subbuf = self.subbuf($from, $elems);
+        Proxy.new(
+            FETCH   => sub ($) { $subbuf },
+            STORE   => sub ($, Blob:D $new) {
+                nqp::splice(nqp::decont(self),nqp::decont($new),$from,$elems)
+            }
+        );
+    }
+
 }
 
 constant buf8 = Buf[uint8];
@@ -712,8 +731,7 @@ multi sub prefix:<~^>(Blob:D \a) {
     nqp::setelems($a,$elems);
 
     my int    $i    = -1;
-    my uint64 $mask = 0xFFFFFFFFFFFFFFFF;
-    nqp::bindpos_i($r,$i,nqp::bitxor_i(nqp::atpos_i($a,$i),$mask))
+    nqp::bindpos_i($r,$i,nqp::bitneg_i(nqp::atpos_i($a,$i)))
       while nqp::islt_i(++$i,$elems);
 
     $r
@@ -802,13 +820,7 @@ multi sub infix:<le> (Blob:D \a, Blob:D \b) { a.COMPARE(b) !=  1      }
 multi sub infix:<ge> (Blob:D \a, Blob:D \b) { a.COMPARE(b) != -1      }
 
 sub subbuf-rw(Buf:D \b, $from = 0, $elems = b.elems - $from) is rw {
-    my Blob $subbuf = b.subbuf($from, $elems);
-    Proxy.new(
-        FETCH   => sub ($) { $subbuf },
-        STORE   => sub ($, Blob:D $new) {
-            nqp::splice(nqp::decont(b),nqp::decont($new),$from,$elems)
-        }
-    );
+    b.subbuf-rw($from, $elems);
 }
 
 # vim: ft=perl6 expandtab sw=4
