@@ -52,39 +52,7 @@ __END__
 ';
     my $perl_wrapper = '#!/usr/bin/env #perl#
 sub MAIN(:$name is copy, :$auth, :$ver, *@, *%) {
-    shift @*ARGS if $name;
-    shift @*ARGS if $auth;
-    shift @*ARGS if $ver;
-    $name //= \'#dist-name#\';
-    my @installations = $*REPO.repo-chain.grep(CompUnit::Repository::Installable);
-    my @binaries = flat @installations.map: { .files(\'bin/#name#\', :$name, :$auth, :$ver) };
-    unless +@binaries {
-        @binaries = flat @installations.map: { .files(\'bin/#name#\', :$name) };
-        if +@binaries {
-            note q:to/SORRY/;
-                ===SORRY!===
-                No candidate found for \'#name#\' that match your criteria.
-                Did you perhaps mean one of these?
-                SORRY
-            my %caps = :name([\'Distribution\', 12]), :auth([\'Author(ity)\', 11]), :ver([\'Version\', 7]);
-            for @binaries -> $dist {
-                for %caps.kv -> $caption, @opts {
-                    @opts[1] = max @opts[1], ($dist{$caption} // \'\').Str.chars
-                }
-            }
-            note \'  \' ~ %caps.values.map({ sprintf(\'%-*s\', .[1], .[0]) }).join(\' | \');
-            for @binaries -> $dist {
-                note \'  \' ~ %caps.kv.map( -> $k, $v { sprintf(\'%-*s\', $v.[1], $dist{$k} // \'\') } ).join(\' | \')
-            }
-        }
-        else {
-            note "===SORRY!===\nNo candidate found for \'#name#\'.\n";
-        }
-        exit 1;
-    }
-
-    %*ENV<PERL6_PROGRAM_NAME> = $*PROGRAM-NAME;
-    exit run($*EXECUTABLE, @binaries.sort(*<ver>).tail.hash.<files><bin/#name#>, @*ARGS).exitcode
+    CompUnit::RepositoryRegistry.run-script("#name#", :dist-name<#dist-name#>, :$name, :$auth, :$ver);
 }';
 
     method !sources-dir() {
@@ -262,8 +230,9 @@ sub MAIN(:$name is copy, :$auth, :$ver, *@, *%) {
             my $handle      = $dist.content($file);
             my $content     = $handle.open(:bin).slurp(:close);
 
-            self!add-short-name($name, $dist, $id, nqp::sha1(
-                $content.decode('iso-8859-1').subst: :g, "\r\n", "\n"));
+            self!add-short-name($name, $dist, $id,
+              nqp::sha1(nqp::join("\n", nqp::split("\r\n",
+                $content.decode('iso-8859-1')))));
             %provides{ $name } = ~$file => {
                 :file($id),
                 :time(try $file.IO.modified.Num),
@@ -290,7 +259,7 @@ sub MAIN(:$name is copy, :$auth, :$ver, *@, *%) {
                     $.prefix.add("$withoutext$be").IO.chmod(0o755);
                 }
             }
-            self!add-short-name($name-path, $dist);
+            self!add-short-name($name-path, $dist, $id);
             %links{$name-path} = $id;
             my $handle  = $dist.content($file);
             my $content = $handle.open.slurp-rest(:bin,:close);
@@ -346,7 +315,7 @@ sub MAIN(:$name is copy, :$auth, :$ver, *@, *%) {
                 }
                 note("Precompiling $id ($source-name)") if $verbose;
                 $precomp.precompile(
-                    $source.IO,
+                    $source,
                     CompUnit::PrecompilationId.new($id),
                     :source-name("$source-file ($source-name)"),
                 );
@@ -402,6 +371,26 @@ sub MAIN(:$name is copy, :$auth, :$ver, *@, *%) {
         unlink( $dist-dir.add($dist.id) )
     }
 
+    method script($file, :$name!, :$auth, :$ver) {
+        my $prefix = self.prefix;
+        my $lookup = $prefix.add('short').add(nqp::sha1($file));
+        return unless $lookup.e;
+
+        # Scripts using this interface could only have been installed long after the introduction of
+        # repo version 1, so we don't have to care about very old repos in this method.
+        my @dists = $lookup.dir.map({
+                my ($ver, $auth, $api, $resource-id) = $_.slurp.split("\n");
+                $resource-id ||= self!read-dist($_.basename)<files>{$file};
+                (id => $_.basename, ver => Version.new( $ver || 0 ), :$auth, :$api, :$resource-id).hash
+            }).grep({
+                $_.<auth> ~~ $auth
+                and $_.<ver> ~~ $ver
+            });
+        for @dists.sort(*.<ver>).reverse {
+            return self!resources-dir.add($_<resource-id>);
+        }
+    }
+
     method files($file, :$name!, :$auth, :$ver) {
         my @candi;
         my $prefix = self.prefix;
@@ -450,7 +439,9 @@ sub MAIN(:$name is copy, :$auth, :$ver, *@, *%) {
                             })
                     ).grep({
                         $_.value<auth> ~~ $spec.auth-matcher
-                        and $_.value<ver> ~~ $spec.version-matcher
+                        and $_.value<ver> ~~ (($spec.version-matcher ~~ Bool)
+                            ?? $spec.version-matcher # fast path for matching Version.new(*)
+                            !! Version.new($spec.version-matcher))
                     });
                 for @dists.sort(*.value<ver>).reverse.map(*.kv) -> ($dist-id, $dist) {
                     return ($dist-id, $dist);
