@@ -2,6 +2,7 @@
 my role Enumeration {
     has $.key;
     has $.value;
+    has int $!index;
 
     method enums() { self.^enum_values.Map }
 
@@ -20,6 +21,14 @@ my role Enumeration {
 
     multi method Numeric(::?CLASS:D:) { $!value.Numeric }
     multi method Int(::?CLASS:D:)     { $!value.Int }
+    multi method Real(::?CLASS:D:)    { $!value.Real }
+
+    multi method WHICH(::?CLASS:D:) {
+        nqp::box_s(
+          nqp::concat(self.^name,nqp::concat("|",$!index)),
+          ObjAt
+        )
+    }
 
     # Make sure we always accept any element of the enumeration
     multi method ACCEPTS(::?CLASS:D: ::?CLASS:U $ --> True) { }
@@ -31,6 +40,33 @@ my role Enumeration {
             ?? $x
             !! self.^enum_from_value($x)
     }
+
+    method pred(::?CLASS:D:) {
+        nqp::if(
+          nqp::getattr_i(self,::?CLASS,'$!index'),
+          nqp::atpos(
+            nqp::getattr(self.^enum_value_list,List,'$!reified'),
+            nqp::sub_i(nqp::getattr_i(self,::?CLASS,'$!index'),1)
+          ),
+          self
+        )
+    }
+    method succ(::?CLASS:D:) {
+        nqp::stmts(
+          (my $values := nqp::getattr(self.^enum_value_list,List,'$!reified')),
+          nqp::if(
+            nqp::islt_i(
+              nqp::getattr_i(self,::?CLASS,'$!index'),
+              nqp::sub_i(nqp::elems($values),1),
+            ),
+            nqp::atpos(
+               $values,
+               nqp::add_i(nqp::getattr_i(self,::?CLASS,'$!index'),1)
+            ),
+            self
+          )
+        )
+    }
 }
 
 # Methods that we also have if the base type of an enumeration is
@@ -77,4 +113,11 @@ Metamodel::EnumHOW.set_composalizer(-> $type, $name, %enum_values {
     $r
 });
 
+# We use this one because, for example, Int:D === Int:D, has an optimization
+# that simply unboxes the values. That's no good for us, since two different
+# Enumertaion:Ds could have the same Int:D value.
+multi infix:<===> (Enumeration:D \a, Enumeration:D \b) {
+    nqp::p6bool(nqp::eqaddr(nqp::decont(a), nqp::decont(b)))
+}
+
 # vim: ft=perl6 expandtab sw=4
