@@ -37,7 +37,7 @@ my class Exception {
         if nqp::isconcrete($!ex) {
             my str $message = nqp::getmessage($!ex);
             $str = nqp::isnull_s($message)
-                ?? "Died with {self.^name}"
+                ?? (try self.?message) // "Died with {self.^name}"
                 !! nqp::p6box_s($message);
             $str ~= "\n";
             try $str ~= self.backtrace
@@ -54,12 +54,6 @@ my class Exception {
         $!ex := nqp::newexception() unless nqp::isconcrete($!ex) and $bt;
         $!bt := $bt; # Even if !$bt
         nqp::setpayload($!ex, nqp::decont(self));
-        my $msg := try self.?message;
-        if defined($msg) {
-            $msg := try ~$msg;
-        }
-        $msg := $msg // "{self.^name} exception produced no message";
-        nqp::setmessage($!ex, nqp::unbox_s($msg));
         nqp::throw($!ex)
     }
     method rethrow(Exception:D:) {
@@ -170,7 +164,7 @@ my class X::Method::NotFound is Exception {
                 when Cool { %suggestions{$_} = 0 for <chars codes>; }
                 default   { %suggestions{$_} = 0 for <elems chars codes>; }
             }
-            
+
         }
         elsif $.method eq 'bytes' {
             %suggestions<encode($encoding).bytes> = 0;
@@ -394,7 +388,7 @@ do {
                 $err.say("===SORRY!===");
                 $err.say($e.Str);
             }
-            Rakudo::Internals.THE_END();
+            nqp::getcurhllsym('&THE_END')();
             CONTROL { when CX::Warn { .resume } }
         }
         if $! {
@@ -660,7 +654,7 @@ my role X::Comp is Exception {
             $r ~= "\n------> $green$.pre$yellow$eject$red$.post$clear" if defined $.pre;
             if $expect && @.highexpect {
                 $r ~= "\n    expecting any of:";
-                for @.highexpect {
+                for flat @.highexpect».list {
                     $r ~= "\n        $_";
                 }
             }
@@ -739,7 +733,7 @@ my class X::Comp::Group is Exception {
 my role X::MOP is Exception { }
 
 my class X::Comp::BeginTime does X::Comp {
-    has $.use-case;
+    has str $.use-case;
     has $.exception;
 
     method message() {
@@ -772,10 +766,12 @@ my class X::Comp::AdHoc is X::AdHoc does X::Comp {
 my class X::Comp::FailGoal does X::Comp {
     has $.dba;
     has $.goal;
+    has $.line-real;
 
     method is-compile-time(--> True) { }
 
-    method message { "Unable to parse expression in $.dba; couldn't find final $.goal" }
+    method message { "Unable to parse expression in $.dba; couldn't find final $.goal"
+                     ~ " (corresponding starter was at line $.line-real)" }
 }
 
 my role X::Syntax does X::Comp { }
@@ -847,6 +843,13 @@ my class X::Worry::P5::LeadingZero is X::Worry::P5 {
         ) ~ '. If you meant to create a string, please add quotation marks.'
     }
 }
+my class X::Worry::Precedence::Range is X::Worry {
+    has $.action;
+    method message {
+"To $!action a range, parenthesize the whole range.
+(Or parenthesize the whole endpoint expression, if you meant that.)"
+    }
+}
 
 my class X::Trait::Invalid is Exception {
     has $.type;       # is, will, of etc.
@@ -1560,7 +1563,7 @@ my class X::Syntax::Perl5Var does X::Syntax {
       '$^O' => 'VM.osname',
       '$^R' => 'an explicit result variable',
       '$^S' => 'context function',
-      '$^T' => '$*INITTIME',
+      '$^T' => '$*INIT-INSTANT',
       '$^V' => '$*PERL.version or $*PERL.compiler.version',
       '$^W' => '$*WARNING',
       '$^X' => '$*EXECUTABLE-NAME',
@@ -1702,6 +1705,25 @@ my class X::Syntax::Regex::SpacesInBareRange does X::Syntax {
     method message { 'Spaces not allowed in bare range.' }
 }
 
+my class X::Syntax::Regex::QuantifierValue does X::Syntax {
+    has $.inf;
+    has $.non-numeric;
+    has $.non-numeric-range;
+    has $.empty-range;
+    method message {
+        $!inf
+          && 'Minimum quantity to match for quantifier cannot be Inf.'
+            ~ ' Did you mean to use + or * quantifiers instead of **?'
+        || $!non-numeric-range
+          && 'Cannot use Range with non-Numeric or NaN end points as quantifier'
+        || $!non-numeric
+          && 'Cannot use non-Numeric or NaN value as quantifier'
+        || $!empty-range
+          && 'Cannot use empty Range as quantifier'
+        || 'Invalid quantifier value'
+    }
+}
+
 my class X::Syntax::Regex::SolitaryQuantifier does X::Syntax {
     method message { 'Quantifier quantifies nothing' }
 }
@@ -1714,6 +1736,10 @@ my class X::Syntax::Regex::SolitaryBacktrackControl does X::Syntax {
     method message { "Backtrack control ':' does not seem to have a preceding atom to control" }
 }
 
+my class X::Syntax::Regex::Alias::LongName does X::Syntax {
+    method message() { "Can only alias to a short name (without '::')"; }
+}
+
 my class X::Syntax::Term::MissingInitializer does X::Syntax {
     method message { 'Term definition requires an initializer' }
 }
@@ -1895,6 +1921,9 @@ my class X::Match::Bool is Exception {
     method message() { "Cannot use Bool as Matcher with '" ~ $.type ~ "'.  Did you mean to use \$_ inside a block?" }
 }
 
+my class X::LibEmpty does X::Comp {
+    method message { q/Repository specification can not be an empty string.  Did you mean 'use lib "."' ?/ }
+}
 my class X::LibNone does X::Comp {
     method message { q/Must specify at least one repository.  Did you mean 'use lib "lib"' ?/ }
 }
@@ -1904,11 +1933,17 @@ my class X::Package::UseLib does X::Comp {
 }
 my class X::Package::Stubbed does X::Comp {
     has @.packages;
-    # TODO: suppress display of line number
     method message() {
         "The following packages were stubbed but not defined:\n    "
         ~ @.packages.join("\n    ");
     }
+
+    # The unnamed named param is here so this candidate, rather than
+    # the one from X::Comp is used. (is it a bug that this is needed?
+    # No idea: https://irclog.perlgeek.de/perl6-dev/2017-09-14#i_15164569 )
+    multi method gist(::?CLASS:D: :$) {
+        $.message;
+    }
 }
 
 my class X::Phaser::PrePost is Exception {
@@ -1984,8 +2019,8 @@ my class X::Str::Trans::InvalidArg is Exception {
 }
 
 my class X::Str::Sprintf::Directives::Count is Exception {
-    has $.args-used;
-    has $.args-have;
+    has int $.args-used;
+    has num $.args-have;
     method message() {
         "Your printf-style directives specify "
         ~ ($.args-used == 1 ?? "1 argument, but "
@@ -2013,6 +2048,22 @@ my class X::Str::Sprintf::Directives::BadType is Exception {
     }
 }
 
+my role X::Encoding is Exception { }
+
+my class X::Encoding::Unknown does X::Encoding {
+    has $.name;
+    method message() {
+        "Unknown string encoding '$.name'"
+    }
+}
+
+my class X::Encoding::AlreadyRegistered does X::Encoding {
+    has $.name;
+    method message() {
+        "An encoding with name '$.name' has already been registered"
+    }
+}
+
 my class X::Range::InvalidArg is Exception {
     has $.got is default(Nil);
     method message() {
@@ -2050,6 +2101,15 @@ my class X::Cannot::New is Exception {
         "Cannot make a {$.class.^name} object using .new";
     }
 }
+my class X::Cannot::Capture is Exception {
+    has $.what;
+    method message() {
+        "Cannot unpack or Capture `$!what.gist()`.\n"
+          ~ "To create a Capture, add parentheses: \\(...)\n"
+          ~ 'If unpacking in a signature, perhaps you needlessly used'
+          ~ ' parentheses? -> ($x) {} vs. -> $x {}';
+    }
+}
 
 my class X::Backslash::UnrecognizedSequence does X::Syntax {
     has $.sequence;
@@ -2199,7 +2259,7 @@ my class X::Assignment::RO is Exception {
     method message {
         "Cannot modify an immutable {$.value.^name} ({$.value.gist})"
     }
-    method typename { $.value.^name } 
+    method typename { $.value.^name }
 }
 
 my class X::Assignment::RO::Comp does X::Comp {
@@ -2329,7 +2389,7 @@ my class X::DateTime::TimezoneClash does X::Temporal {
 my class X::DateTime::InvalidDeltaUnit does X::Temporal {
     has $.unit;
     method message() {
-        "Cannnot use unit $.unit with Date.delta";
+        "Cannot use unit $.unit with Date.delta";
     }
 }
 
@@ -2365,15 +2425,17 @@ my class X::Import::Positional is Exception {
     }
 }
 
-my class X::Numeric::Real is Exception {
+my class X::Numeric::CannotConvert is Exception {
     has $.target;
     has $.reason;
     has $.source;
 
     method message() {
-        "Can not convert $.source to {$.target.^name}: $.reason";
+        "Cannot convert $!source to {$!target // $!target.perl}: $!reason";
     }
+
 }
+my class X::Numeric::Real is X::Numeric::CannotConvert {}
 
 my class X::Numeric::DivideByZero is Exception {
     has $.using;
@@ -2382,7 +2444,7 @@ my class X::Numeric::DivideByZero is Exception {
     method message() {
         "Attempt to divide{$.numerator ?? " $.numerator" !! ''} by zero"
           ~ ( $.using ?? " using $.using" !! '' )
-          ~ ( $_ with $.details );
+          ~ ( " $_" with $.details );
     }
 }
 
@@ -2403,7 +2465,7 @@ my class X::Numeric::Confused is Exception {
         ~ (
             "\n(If you really wanted to convert {$.num.perl} to a base-$.base"
                 ~ " string, use {$.num.perl}.base($.base) instead.)"
-            if $.num.perl.^can('base')
+            if $.num.^can('base')
         );
     }
 }
@@ -2446,8 +2508,8 @@ my class X::Multi::NoMatch is Exception {
         my @un-rw-cand;
         if first / 'is rw' /, @cand {
             my $rw-capture = Capture.new(
-                :list( $!capture.list.map({ $ = $_ })                  ),
-                :hash( $!capture.hash.map({ .key => $ = .value }).hash ),
+                :list( $!capture.list.map({ my $ = $_ })                  ),
+                :hash( $!capture.hash.map({ .key => my $ = .value }).hash ),
             );
             @un-rw-cand = $.dispatcher.dispatchees».signature.grep({
                 $rw-capture ~~ $^cand
