@@ -204,22 +204,21 @@ my class Hash { # declared in BOOTSTRAP
     }
 
     multi method perl(Hash:D \SELF:) {
-        SELF.perlseen('Hash', {
+        SELF.perlseen(self.^name, {
             '$' x nqp::iscont(SELF)  # self is always deconted
-            ~ '{' ~ self.pairs.sort.map({.perl}).join(', ') ~ '}'
+            ~ '{' ~ self.sort.map({.perl}).join(', ') ~ '}'
         })
     }
 
     multi method gist(Hash:D:) {
-        self.gistseen('Hash', {
+        self.gistseen(self.^name, {
             '{' ~
-            self.pairs.sort.map( -> $elem {
-                given ++$ {
-                    when 101 { '...' }
-                    when 102 { last }
-                    default  { $elem.gist }
-                }
-            } ).join(', ')
+            self.sort.map({
+                state $i = 0;
+                ++$i == 101 ?? '...'
+                    !! $i == 102 ?? last()
+                        !! .gist
+            }).join(', ')
             ~ '}'
         })
     }
@@ -523,11 +522,13 @@ my class Hash { # declared in BOOTSTRAP
         }
         multi method perl(::?CLASS:D \SELF:) {
             SELF.perlseen('Hash', {
-                self.elems
-                  ?? "(my {TValue.perl} % = {
-                       self.pairs.sort.map({.perl}).join(', ')
-                     })"
-                  !! "(my {TValue.perl} %)"
+                '$' x nqp::iscont(SELF)  # self is always deconted
+                ~ (self.elems
+                   ?? "(my {TValue.perl} % = {
+                        self.sort.map({.perl}).join(', ')
+                       })"
+                   !! "(my {TValue.perl} %)"
+                  )
             })
         }
     }
@@ -641,6 +642,37 @@ my class Hash { # declared in BOOTSTRAP
               TValue
             )
         }
+
+        method FLATTENABLE_HASH() {
+            nqp::stmts(
+              (my $flattened := nqp::hash),
+              nqp::if(
+                (my $raw := nqp::getattr(self,Map,'$!storage'))
+                 && (my $iter := nqp::iterator($raw)),
+                nqp::while(
+                  $iter,
+                  nqp::bindkey(
+                    $flattened,
+                    nqp::if(
+                      nqp::istype(
+                        (my $key := nqp::getattr(
+                          nqp::iterval(nqp::shift($iter)),
+                          Pair,
+                          '$!key'
+                        )),
+                        Str,
+                      ),
+                      $key,
+                      $key.Str
+                    ),
+                    nqp::getattr(nqp::iterval($iter),Pair,'$!value')
+                  )
+                )
+              ),
+              $flattened
+            )
+        }
+
         method IterationBuffer() {
             nqp::stmts(
               (my $buffer := nqp::create(IterationBuffer)),
@@ -708,17 +740,82 @@ my class Hash { # declared in BOOTSTRAP
                  }
             }.new(self))
         }
+        multi method roll(::?CLASS:D:) {
+            nqp::if(
+              (my $raw := nqp::getattr(self,Map,'$!storage')) && nqp::elems($raw),
+              nqp::stmts(
+                (my int $i = nqp::add_i(nqp::elems($raw).rand.floor,1)),
+                (my $iter := nqp::iterator($raw)),
+                nqp::while(
+                  nqp::shift($iter) && ($i = nqp::sub_i($i,1)),
+                  nqp::null
+                ),
+                nqp::iterval($iter)
+              ),
+              Nil
+            )
+        }
+        multi method roll(::?CLASS:D: Callable:D $calculate) {
+            self.roll( $calculate(self.elems) )
+        }
+        multi method roll(::?CLASS:D: Whatever $) { self.roll(Inf) }
+        multi method roll(::?CLASS:D: $count) {
+            Seq.new(nqp::if(
+              (my $raw := nqp::getattr(self,Map,'$!storage'))
+                && nqp::elems($raw) && $count > 0,
+              class :: does Iterator {
+                  has $!storage;
+                  has $!keys;
+                  has $!count;
+
+                  method !SET-SELF(\hash,\count) {
+                      nqp::stmts(
+                        ($!storage := nqp::getattr(hash,Map,'$!storage')),
+                        ($!count = $count),
+                        (my $iter := nqp::iterator($!storage)),
+                        ($!keys := nqp::list_s),
+                        nqp::while(
+                          $iter,
+                          nqp::push_s($!keys,nqp::iterkey_s(nqp::shift($iter)))
+                        ),
+                        self
+                      )
+                  }
+                  method new(\h,\c) { nqp::create(self)!SET-SELF(h,c) }
+                  method pull-one() {
+                      nqp::if(
+                        $!count,
+                        nqp::stmts(
+                          --$!count,  # must be HLL to handle Inf
+                          nqp::atkey(
+                            $!storage,
+                            nqp::atpos_s($!keys,nqp::elems($!keys).rand.floor)
+                          )
+                        ),
+                        IterationEnd
+                      )
+                  }
+                  method is-lazy() { $!count == Inf }
+              }.new(self,$count),
+              Rakudo::Iterator.Empty
+            ))
+        }
         multi method perl(::?CLASS:D \SELF:) {
             SELF.perlseen('Hash', {
                 my $TKey-perl   := TKey.perl;
                 my $TValue-perl := TValue.perl;
                 $TKey-perl eq 'Any' && $TValue-perl eq 'Mu'
-                  ?? ':{' ~ SELF.pairs.sort.map({.perl}).join(', ') ~ '}'
-                  !! self.elems
-                        ?? "(my $TValue-perl %\{$TKey-perl\} = {
-                          self.pairs.sort.map({.perl}).join(', ')
-                        })"
-                        !! "(my $TValue-perl %\{$TKey-perl\})"
+                  ?? ( '$(' x nqp::iscont(SELF)
+                        ~ ':{' ~ SELF.sort.map({.perl}).join(', ') ~ '}'
+                        ~ ')' x nqp::iscont(SELF)
+                     )
+                  !! '$' x nqp::iscont(SELF)
+                     ~ (self.elems
+                          ?? "(my $TValue-perl %\{$TKey-perl\} = {
+                                self.sort.map({.perl}).join(', ')
+                             })"
+                          !! "(my $TValue-perl %\{$TKey-perl\})"
+                     )
             })
         }
 
@@ -739,17 +836,8 @@ my class Hash { # declared in BOOTSTRAP
               !! nqp::create(Capture)
         }
         method Map() { self.pairs.Map }
-
-        sub SETIFY(\objecthash, \type) {
-            nqp::create(type).SET-SELF(
-              Rakudo::QuantHash.ADD-OBJECTHASH-TO-SET(
-                nqp::create(Rakudo::Internals::IterationSet), objecthash
-              )
-            )
-        }
-        multi method Set(::?CLASS:D:)     { SETIFY(self,Set)     }
-        multi method SetHash(::?CLASS:D:) { SETIFY(self,SetHash) }
     }
+
     method ^parameterize(Mu:U \hash, Mu:U \t, |c) {
         if c.elems == 0 {
             my $what := hash.^mixin(TypedHash[t]);
