@@ -1,5 +1,4 @@
 my class IO::Path { ... }
-my class IO::Special { ... }
 my class Proc { ... }
 
 my class IO::Handle {
@@ -9,15 +8,53 @@ my class IO::Handle {
     has $.nl-in = ["\x0A", "\r\n"];
     has Str:D $.nl-out is rw = "\n";
     has Str $.encoding;
-    has Rakudo::Internals::VMBackedDecoder $!decoder;
+    has Encoding::Decoder $!decoder;
+    has Encoding::Encoder $!encoder;
+    has int $!out-buffer;
 
-    submethod TWEAK (:$encoding, :$bin) {
+    submethod TWEAK (:$encoding, :$bin, IO() :$!path = Nil) {
         nqp::if(
           $bin,
           nqp::isconcrete($encoding) && X::IO::BinaryAndEncoding.new.throw,
           $!encoding = $encoding || 'utf8')
     }
 
+#?if moar
+    # Make sure we close any open files on exit
+    my $opened := nqp::list;
+    my $opened-locker = Lock.new;
+    method !remember-to-close(--> Nil) {
+        $opened-locker.protect: {
+            nqp::stmts(
+              nqp::if(
+                nqp::isge_i(
+                  (my int $fileno = nqp::filenofh($!PIO)),
+                  (my int $elems = nqp::elems($opened))
+                ),
+                nqp::setelems($opened,nqp::add_i($elems,1024))
+              ),
+              nqp::bindpos($opened,$fileno,$!PIO)
+            )
+        }
+    }
+    method !forget-about-closing(int $fileno --> Nil) {
+        $opened-locker.protect: {
+            nqp::bindpos($opened,$fileno,nqp::null)
+        }
+    }
+    method !close-all-open-handles() {
+        my int $i = 2;
+        my int $elems = nqp::elems($opened);
+        nqp::while(
+          nqp::islt_i(($i = nqp::add_i($i,1)),$elems),
+          nqp::unless(
+            nqp::isnull(my $PIO := nqp::atpos($opened,$i)),
+            nqp::closefh($PIO)
+          )
+        )
+    }
+#?endif
+
     method open(IO::Handle:D:
       :$r, :$w, :$x, :$a, :$update,
       :$rw, :$rx, :$ra,
@@ -31,7 +68,16 @@ my class IO::Handle {
       :$chomp = $!chomp,
       :$nl-in is copy = $!nl-in,
       Str:D :$nl-out is copy = $!nl-out,
+      :$buffer,
+      :$out-buffer is copy,
     ) {
+        nqp::if(
+            $buffer.DEFINITE,
+            nqp::stmts(
+              ($out-buffer = $buffer),
+              DEPRECATED ':out-buffer argument to control handle buffering',
+                '2017.09.455.g.2.fba.0.ba.0.d', '2018.01'));
+
         nqp::if(
           $bin,
           nqp::stmts(
@@ -124,10 +170,13 @@ my class IO::Handle {
             $!chomp = $chomp;
             $!nl-out = $nl-out;
             if nqp::isconcrete($enc) {
-                $!encoding = Rakudo::Internals.NORMALIZE_ENCODING($enc);
-                $!decoder := Rakudo::Internals::VMBackedDecoder.new($!encoding, :translate-nl);
+                my $encoding = Encoding::Registry.find($enc);
+                $!decoder := $encoding.decoder(:translate-nl);
                 $!decoder.set-line-separators(($!nl-in = $nl-in).list);
+                $!encoder := $encoding.encoder(:translate-nl);
+                $!encoding = $encoding.name;
             }
+            self!set-out-buffer-size($out-buffer);
             return self;
         }
 
@@ -160,18 +209,39 @@ my class IO::Handle {
                     )
                 ),
             );
+#?if moar
+            self!remember-to-close;
+#?endif
         }
 
         $!chomp = $chomp;
         $!nl-out = $nl-out;
         if nqp::isconcrete($enc) {
-            $!encoding = Rakudo::Internals.NORMALIZE_ENCODING($enc);
-            $!decoder := Rakudo::Internals::VMBackedDecoder.new($!encoding, :translate-nl);
+            my $encoding = Encoding::Registry.find($enc);
+            $!decoder := $encoding.decoder(:translate-nl);
             $!decoder.set-line-separators(($!nl-in = $nl-in).list);
+            $!encoder := $encoding.encoder(:translate-nl);
+            $!encoding = $encoding.name;
         }
+        self!set-out-buffer-size($out-buffer);
         self;
     }
 
+    method out-buffer is rw {
+        Proxy.new: :FETCH{ $!out-buffer }, STORE => -> $, \buffer {
+            self!set-out-buffer-size: buffer;
+        }
+    }
+
+    method !set-out-buffer-size($buffer is copy) {
+        $buffer //= !nqp::isttyfh($!PIO);
+        $!out-buffer = nqp::istype($buffer, Bool)
+            ?? ($buffer ?? 8192 !! 0)
+            !! $buffer.Int;
+        nqp::setbuffersizefh($!PIO, $!out-buffer);
+        $!out-buffer
+    }
+
     method nl-in is rw {
         Proxy.new(
           FETCH => {
@@ -189,8 +259,16 @@ my class IO::Handle {
         nqp::if(
           nqp::defined($!PIO),
           nqp::stmts(
+#?if !moar
             nqp::closefh($!PIO), # TODO: catch errors
             $!PIO := nqp::null
+#?endif
+#?if moar
+            (my int $fileno = nqp::filenofh($!PIO)),
+            nqp::closefh($!PIO), # TODO: catch errors
+            ($!PIO := nqp::null),
+            self!forget-about-closing($fileno)
+#?endif
           )
         )
     }
@@ -234,12 +312,7 @@ my class IO::Handle {
 
     method getc(IO::Handle:D:) {
         $!decoder or die X::IO::BinaryMode.new(:trying<getc>);
-        $!decoder.consume-exactly-chars(1) || self!getc-slow-path()
-    }
-
-    method !getc-slow-path() {
-        $!decoder.add-bytes(self.read-internal(0x100000));
-        $!decoder.consume-exactly-chars(1) // $!decoder.consume-all-chars() || Nil
+        $!decoder.consume-exactly-chars(1) || (self!readchars-slow-path(1) || Nil)
     }
 
     # XXX TODO: Make these routine read handle lazily when we have Cat type
@@ -469,7 +542,7 @@ my class IO::Handle {
                     last if nqp::isconcrete($result);
                 }
                 else {
-                    $result := $!decoder.consume-all-chars();
+                    $result := $!decoder.consume-exactly-chars($chars, :eof);
                     last;
                 }
             }
@@ -517,7 +590,7 @@ my class IO::Handle {
 
             # Freshen decoder, so we won't have stuff left over from earlier reads
             # that were in the wrong place.
-            $!decoder := Rakudo::Internals::VMBackedDecoder.new($!encoding, :translate-nl);
+            $!decoder := Encoding::Registry.find($!encoding).decoder(:translate-nl);
             $!decoder.set-line-separators($!nl-in.list);
         }
         nqp::seekfh($!PIO, $offset - $rewind, +$whence);
@@ -546,8 +619,14 @@ my class IO::Handle {
     method lock(IO::Handle:D:
         Bool:D :$non-blocking = False, Bool:D :$shared = False --> True
     ) {
+#?if moar
+        self!forget-about-closing(nqp::filenofh($!PIO));
+#?endif
         nqp::lockfh($!PIO, 0x10*$non-blocking + $shared);
         CATCH { default {
+#?if moar
+            self!remember-to-close;
+#?endif
             fail X::IO::Lock.new: :os-error(.Str),
                 :lock-type( 'non-' x $non-blocking ~ 'blocking, '
                     ~ ($shared ?? 'shared' !! 'exclusive') );
@@ -555,6 +634,9 @@ my class IO::Handle {
     }
 
     method unlock(IO::Handle:D: --> True) {
+#?if moar
+        self!remember-to-close;
+#?endif
         nqp::unlockfh($!PIO);
     }
 
@@ -565,7 +647,7 @@ my class IO::Handle {
     proto method print(|) { * }
     multi method print(IO::Handle:D: Str:D \x --> True) {
         $!decoder or die X::IO::BinaryMode.new(:trying<print>);
-        self.write-internal(x.encode($!encoding, :translate-nl));
+        self.write-internal($!encoder.encode-chars(x));
     }
     multi method print(IO::Handle:D: **@list is raw --> True) { # is raw gives List, which is cheaper
         self.print(@list.join);
@@ -574,17 +656,22 @@ my class IO::Handle {
     proto method put(|) { * }
     multi method put(IO::Handle:D: Str:D \x --> True) {
         $!decoder or die X::IO::BinaryMode.new(:trying<put>);
-        self.write-internal(nqp::concat(nqp::unbox_s(x),
-            nqp::unbox_s($!nl-out)).encode($!encoding, :translate-nl))
+        self.write-internal($!encoder.encode-chars(
+            nqp::concat(nqp::unbox_s(x), nqp::unbox_s($!nl-out))))
     }
     multi method put(IO::Handle:D: **@list is raw --> True) { # is raw gives List, which is cheaper
         self.put(@list.join);
     }
 
+    multi method say(IO::Handle:D: Str:D $x --> True) {
+        $!decoder or die X::IO::BinaryMode.new(:trying<say>);
+        self.write-internal($!encoder.encode-chars(
+            nqp::concat(nqp::unbox_s($x), nqp::unbox_s($!nl-out))));
+    }
     multi method say(IO::Handle:D: \x --> True) {
         $!decoder or die X::IO::BinaryMode.new(:trying<say>);
-        self.write-internal(nqp::concat(nqp::unbox_s(x.gist),
-            nqp::unbox_s($!nl-out)).encode($!encoding, :translate-nl))
+        self.write-internal($!encoder.encode-chars(
+            nqp::concat(nqp::unbox_s(x.gist), nqp::unbox_s($!nl-out))))
     }
     multi method say(IO::Handle:D: |) {
         $!decoder or die X::IO::BinaryMode.new(:trying<say>);
@@ -597,7 +684,7 @@ my class IO::Handle {
 
     method print-nl(IO::Handle:D: --> True) {
         $!decoder or die X::IO::BinaryMode.new(:trying<print-nl>);
-        self.write-internal($!nl-out.encode($!encoding, :translate-nl));
+        self.write-internal($!encoder.encode-chars($!nl-out));
     }
 
     proto method slurp-rest(|) { * }
@@ -624,23 +711,29 @@ my class IO::Handle {
         self!slurp-all-chars()
     }
 
-    method slurp(IO::Handle:D: :$close) {
-        my $res;
-        nqp::if(
-          $!decoder,
-          ($res := self!slurp-all-chars()),
-          nqp::stmts(
-            ($res := buf8.new),
+    method slurp(IO::Handle:D: :$close, :$bin) {
+        nqp::stmts(
+          (my $res),
+          nqp::if(
+            $!decoder,
+            nqp::if(
+              $bin,
+              nqp::stmts(
+                ($res := buf8.new),
+                nqp::if(
+                  $!decoder.bytes-available,
+                  $res.append($!decoder.consume-exactly-bytes(
+                    $!decoder.bytes-available)))),
+              ($res := self!slurp-all-chars())),
+            ($res := buf8.new)),
+          nqp::if(
+            nqp::isfalse($!decoder) || $bin,
             nqp::while(
               nqp::elems(my $buf := self.read-internal(0x100000)),
-              $res.append($buf)
-            )
-          )
-        );
-
-        # don't sink result of .close; it might be a failed Proc
-        $ = self.close if $close;
-        $res
+              $res.append($buf))),
+          # don't sink result of .close; it might be a failed Proc
+          nqp::if($close, my $ = self.close),
+          $res)
     }
 
     method !slurp-all-chars() {
@@ -684,7 +777,6 @@ my class IO::Handle {
                 $_ = Nil;
             }
             else {
-                $_ = Rakudo::Internals.NORMALIZE_ENCODING(.Str);
                 return $!encoding if $!encoding && $!encoding eq $_;
             }
         }
@@ -695,15 +787,18 @@ my class IO::Handle {
             my $available = $!decoder.bytes-available;
             with $new-encoding {
                 my $prev-decoder := $!decoder;
-                $!decoder := Rakudo::Internals::VMBackedDecoder.new($new-encoding, :translate-nl);
+                my $encoding = Encoding::Registry.find($new-encoding);
+                $!decoder := $encoding.decoder(:translate-nl);
                 $!decoder.set-line-separators($!nl-in.list);
                 $!decoder.add-bytes($prev-decoder.consume-exactly-bytes($available))
                     if $available;
-                $!encoding = $new-encoding;
+                $!encoder := $encoding.encoder(:translate-nl);
+                $!encoding = $encoding.name;
             }
             else {
                 nqp::seekfh($!PIO, -$available, SeekFromCurrent) if $available;
-                $!decoder := Rakudo::Internals::VMBackedDecoder;
+                $!decoder := Encoding::Decoder;
+                $!encoder := Encoding::Encoder;
                 $!encoding = Nil;
                 Nil
             }
@@ -711,9 +806,11 @@ my class IO::Handle {
         else {
             # No previous decoder; make a new one if needed, otherwise no change.
             with $new-encoding {
-                $!decoder := Rakudo::Internals::VMBackedDecoder.new($new-encoding, :translate-nl);
+                my $encoding = Encoding::Registry.find($new-encoding);
+                $!decoder := $encoding.decoder(:translate-nl);
                 $!decoder.set-line-separators($!nl-in.list);
-                $!encoding = $new-encoding;
+                $!encoder := $encoding.encoder(:translate-nl);
+                $!encoding = $encoding.name;
             }
             else {
                 Nil
@@ -722,11 +819,21 @@ my class IO::Handle {
     }
 
     submethod DESTROY(IO::Handle:D:) {
+        # Close handles with any file descriptor larger than 2. Those below
+        # are our $*IN, $*OUT, and $*ERR, and we don't want them closed
+        # implicitly via DESTROY, since you can't get them back again.
         nqp::if(
-          nqp::defined($!PIO),
+          nqp::defined($!PIO)
+            && nqp::isgt_i((my int $fileno = nqp::filenofh($!PIO)), 2),
           nqp::stmts(
             nqp::closefh($!PIO),  # don't bother checking for errors
+#?if !moar
             $!PIO := nqp::null
+#?endif
+#?if moar
+            ($!PIO := nqp::null),
+            self!forget-about-closing($fileno)
+#?endif
           )
         )
     }
