@@ -1,5 +1,3 @@
-my class Instant { ... }
-
 my class IO::Path is Cool does IO {
     has IO::Spec $.SPEC;
     has Str      $.CWD;
@@ -13,7 +11,7 @@ my class IO::Path is Cool does IO {
     }
 
     submethod BUILD(:$!path!, :$!SPEC!, :$!CWD! --> Nil) {
-        nqp::unless($!path,
+        nqp::unless(nqp::chars($!path), # could be an IntStr, so check chars
             die "Must specify something as a path: did you mean '.' for the current directory?"
         );
         nqp::if(
@@ -24,13 +22,13 @@ my class IO::Path is Cool does IO {
     }
 
     method !new-from-absolute-path($path, :$SPEC = $*SPEC, Str() :$CWD = $*CWD) {
-        method !set-absolute() {
-            $!is-absolute = True;
-            $!abspath := $path;
-            self;
-        }
+        self.bless(:$path, :$SPEC, :$CWD)!set-absolute($path);
+    }
 
-        self.bless(:$path, :$SPEC, :$CWD)!set-absolute;
+    method !set-absolute($path) {
+        $!is-absolute = True;
+        $!abspath := $path;
+        self;
     }
 
     proto method new(|) {*}
@@ -328,8 +326,13 @@ my class IO::Path is Cool does IO {
         $resolved = $sep unless nqp::chars($resolved);
         IO::Path!new-from-absolute-path($resolved,:$!SPEC,:CWD($sep));
     }
-
-    method parent(IO::Path:D:) {    # XXX needs work
+    proto method parent(|) { * }
+    multi method parent(IO::Path:D: UInt:D $depth) {
+        my $io = self;
+        $io .= parent xx $depth;
+        $io;
+    }
+    multi method parent(IO::Path:D:) {    # XXX needs work
         my $curdir := $!SPEC.curdir;
         my $updir  := $!SPEC.updir;
 
@@ -408,6 +411,9 @@ my class IO::Path is Cool does IO {
         );
         self.chdir: $path, |$test.words.map(* => True).Hash;
     }
+    multi method chdir(IO::Path:D: IO $path, |c) {
+        self.chdir: $path.absolute, |c
+    }
     multi method chdir(
         IO::Path:D: Str() $path is copy, :$d = True, :$r, :$w, :$x,
     ) {
@@ -609,7 +615,8 @@ my class IO::Path is Cool does IO {
             $handle,
             nqp::stmts(
                 (my $blob := $handle.slurp(:close)),
-                nqp::if($bin, $blob, $blob.decode($enc || 'utf-8').subst("\r\n", "\n", :g))
+                nqp::if($bin, $blob, nqp::join("\n",
+                  nqp::split("\r\n", $blob.decode: $enc || 'utf-8')))
             ))
     }
 
