@@ -48,6 +48,7 @@ my class Instant is Cool does Real {
 
     method Date(Instant:D:)     { Date.new(self)     }
     method DateTime(Instant:D:) { DateTime.new(self) }
+    method Instant() { self }
 
 #    TODO: should be the new .gist, probably
 #    method Str() {
@@ -70,6 +71,9 @@ multi sub infix:«==»(Instant:D $a, Instant:D $b) {
 multi sub infix:«!=»(Instant:D $a, Instant:D $b) {
     $a.tai != $b.tai
 }
+multi sub infix:«≠»(Instant:D $a, Instant:D $b) {
+    $a.tai ≠ $b.tai
+}
 
 multi sub infix:«<»(Instant:D $a, Instant:D $b) {
     $a.tai < $b.tai
@@ -82,10 +86,16 @@ multi sub infix:«>»(Instant:D $a, Instant:D $b) {
 multi sub infix:«<=»(Instant:D $a, Instant:D $b) {
     $a.tai <= $b.tai
 }
+multi sub infix:«≤»(Instant:D $a, Instant:D $b) {
+    $a.tai ≤ $b.tai
+}
 
 multi sub infix:«>=»(Instant:D $a, Instant:D $b) {
     $a.tai >= $b.tai
 }
+multi sub infix:«≥»(Instant:D $a, Instant:D $b) {
+    $a.tai ≥ $b.tai
+}
 
 multi sub infix:<+>(Instant:D $a, Real:D $b) {
     nqp::create(Instant).SET-SELF($a.tai + $b.Rat)
@@ -116,10 +126,16 @@ sub term:<now>() {
     )
 }
 
-Rakudo::Internals.REGISTER-DYNAMIC: '$*INITTIME', {
-    PROCESS::<$INITTIME> := nqp::create(Instant).SET-SELF(
+Rakudo::Internals.REGISTER-DYNAMIC: '$*INIT-INSTANT', {
+    PROCESS::<$INIT-INSTANT> := nqp::create(Instant).SET-SELF(
       Rakudo::Internals.tai-from-posix(Rakudo::Internals.INITTIME,0).Rat
     )
 }
+Rakudo::Internals.REGISTER-DYNAMIC: '$*INITTIME', {
+    my ($file, $line) = .file, .line with callframe 3;
+    DEPRECATED('$*INIT-INSTANT', '2017.09.84.gb.02.da.4.d.1.a', '2018.08',
+        :what<$*INITTIME>, :$file, :$line);
+    $*INIT-INSTANT
+}
 
 # vim: ft=perl6 expandtab sw=4
