@@ -1,4 +1,5 @@
 my class Rat { ... }
+my class X::Cannot::Capture       { ... }
 my class X::Numeric::DivideByZero { ... }
 my class X::NYI::BigInt { ... }
 
@@ -22,11 +23,18 @@ my class Int does Real { # declared in BOOTSTRAP
           ObjAt
         )
     }
-    multi method new($value) {
-        # clone to ensure we return a new object for any cached
-        # numeric constants
-        $value.Int.clone;
+
+    proto method new(|) {*}
+    multi method new(      \value) { self.new: value.Int }
+    multi method new(int   \value) {
+        # rebox the value, so we get rid of any potential mixins
+        nqp::div_I(nqp::decont(value), 1, self)
     }
+    multi method new(Int:D \value = 0) {
+        # rebox the value, so we get rid of any potential mixins
+        nqp::div_I(nqp::decont(value), 1, self)
+    }
+
     multi method perl(Int:D:) {
         self.Str;
     }
@@ -34,6 +42,8 @@ my class Int does Real { # declared in BOOTSTRAP
         nqp::p6bool(nqp::bool_I(self));
     }
 
+    method Capture() { die X::Cannot::Capture.new: :what(self) }
+
     method Int() { self }
 
     multi method Str(Int:D:) {
@@ -62,7 +72,7 @@ my class Int does Real { # declared in BOOTSTRAP
     method chr(Int:D:) {
         nqp::if(
           nqp::isbig_I(self),
-          (die "chr codepoint too large: {self}"),
+            die("chr codepoint %i (0x%X) is too large".sprintf(self, self)),
           nqp::p6box_s(nqp::chr(nqp::unbox_i(self)))
         )
     }
@@ -344,9 +354,8 @@ multi sub infix:<==>(int $a, int $b) {
     nqp::p6bool(nqp::iseq_i($a, $b))
 }
 
-multi sub infix:<!=>(int $a, int $b) {
-    nqp::p6bool(nqp::isne_i($a, $b))
-}
+multi sub infix:<!=>(int $a, int $b) { nqp::p6bool(nqp::isne_i($a, $b)) }
+multi sub infix:<≠> (int $a, int $b) { nqp::p6bool(nqp::isne_i($a, $b)) }
 
 multi sub infix:«<»(Int:D \a, Int:D \b) {
     nqp::p6bool(nqp::islt_I(nqp::decont(a), nqp::decont(b)))
@@ -361,6 +370,12 @@ multi sub infix:«<=»(Int:D \a, Int:D \b) {
 multi sub infix:«<=»(int $a, int $b) {
     nqp::p6bool(nqp::isle_i($a, $b))
 }
+multi sub infix:«≤»(Int:D \a, Int:D \b) {
+    nqp::p6bool(nqp::isle_I(nqp::decont(a), nqp::decont(b)))
+}
+multi sub infix:«≤»(int $a, int $b) {
+    nqp::p6bool(nqp::isle_i($a, $b))
+}
 
 multi sub infix:«>»(Int:D \a, Int:D \b) {
     nqp::p6bool(nqp::isgt_I(nqp::decont(a), nqp::decont(b)))
@@ -375,6 +390,12 @@ multi sub infix:«>=»(Int:D \a, Int:D \b) {
 multi sub infix:«>=»(int $a, int $b) {
     nqp::p6bool(nqp::isge_i($a, $b))
 }
+multi sub infix:«≥»(Int:D \a, Int:D \b) {
+    nqp::p6bool(nqp::isge_I(nqp::decont(a), nqp::decont(b)))
+}
+multi sub infix:«≥»(int $a, int $b) {
+    nqp::p6bool(nqp::isge_i($a, $b))
+}
 
 multi sub infix:<+|>(Int:D \a, Int:D \b) {
     nqp::bitor_I(nqp::decont(a), nqp::decont(b), Int)
@@ -398,9 +419,6 @@ multi sub infix:<+^>(Int:D \a, Int:D \b) {
 #}
 
 multi sub infix:«+<»(Int:D \a, Int:D \b --> Int:D) {
-#?if moar
-    my $ignore = nqp::ctx(); # <-- kludge for RT#131306
-#?endif
     nqp::bitshiftl_I(nqp::decont(a), nqp::unbox_i(b), Int)
 }
 #multi sub infix:«+<»(int $a, int $b) { RT#128655
@@ -408,9 +426,6 @@ multi sub infix:«+<»(Int:D \a, Int:D \b --> Int:D) {
 #}
 
 multi sub infix:«+>»(Int:D \a, Int:D \b --> Int:D) {
-#?if moar
-    my $ignore = nqp::ctx(); # <-- kludge for RT#131306
-#?endif
     nqp::bitshiftr_I(nqp::decont(a), nqp::unbox_i(b), Int)
 }
 #multi sub infix:«+>»(int $a, int $b) { RT#128655
