@@ -1,7 +1,4 @@
 # TODO:
-# * $?USAGE
-#   * Create $?USAGE at compile time
-#   * Make $?USAGE available globally
 # * Command-line parsing
 #   * Allow both = and space before argument of double-dash args
 #   * Comma-separated list values
@@ -13,7 +10,8 @@ my sub MAIN_HELPER($retval = 0) {
     my $m = callframe(1).my<&MAIN>;
     return $retval unless $m;
 
-    my $no-named-after = !$*MAIN-ALLOW-NAMED-ANYWHERE;
+    my %SUB-MAIN-OPTS  := %*SUB-MAIN-OPTS // {};
+    my $no-named-after := nqp::isfalse(%SUB-MAIN-OPTS<named-anywhere>);
 
     sub thevalue(\a) {
         ((my $type := ::(a)) andthen Metamodel::EnumHOW.ACCEPTS($type.HOW))
@@ -183,9 +181,12 @@ my sub MAIN_HELPER($retval = 0) {
 
     # Generate default $?USAGE message
     my $usage;
-    my $?USAGE := Proxy.new(
+    my $*USAGE := Proxy.new(
         FETCH => -> | { $usage || ($usage = gen-usage()) },
-        STORE => -> | { }
+        STORE => -> | {
+            die 'Cannot assign to $*USAGE. Please use `sub USAGE {}` to '
+                ~ 'output custom usage message'
+        }
     );
 
     # Get a list of candidates that match according to the dispatcher
@@ -209,17 +210,13 @@ my sub MAIN_HELPER($retval = 0) {
     # We could not find a user defined USAGE sub!
     # Let's display the default USAGE message
     if $n<help> {
-        $*OUT.say($?USAGE);
-        exit 1;
+        $*OUT.say($*USAGE);
+        exit 0;
     }
     else {
-        $*ERR.say($?USAGE);
+        $*ERR.say($*USAGE);
         exit 2;
     }
 }
 
-Rakudo::Internals.REGISTER-DYNAMIC: '$*MAIN-ALLOW-NAMED-ANYWHERE', {
-    PROCESS::<$MAIN-ALLOW-NAMED-ANYWHERE> := 0;
-}
-
 # vim: ft=perl6 expandtab sw=4
