@@ -1,11 +1,9 @@
+my class X::Cannot::Lazy             { ... }
 my class X::Constructor::Positional  { ... }
 my class X::Method::NotFound         { ... }
 my class X::Method::InvalidQualifier { ... }
 my class X::Attribute::Required      { ... }
 
-# We use a sentinel value to mark the end of an iteration.
-my constant IterationEnd = nqp::create(Mu);
-
 my class Mu { # declared in BOOTSTRAP
 
     method self { self }
@@ -111,8 +109,15 @@ Perhaps it can be found at https://docs.perl6.org/type/$name"
     }
 
     proto method new(|) { * }
-    multi method new(*%) {
-        nqp::invokewithcapture(nqp::findmethod(self, 'bless'), nqp::usecapture())
+    multi method new(*%attrinit) {
+        nqp::if(
+          nqp::eqaddr(
+            (my $bless := nqp::findmethod(self,'bless')),
+            nqp::findmethod(Mu,'bless')
+          ),
+          nqp::create(self).BUILDALL(Empty, %attrinit),
+          nqp::invokewithcapture($bless,nqp::usecapture)
+        )
     }
     multi method new($, *@) {
         X::Constructor::Positional.new(:type( self )).throw();
@@ -126,372 +131,407 @@ Perhaps it can be found at https://docs.perl6.org/type/$name"
     }
 
     method bless(*%attrinit) {
-        nqp::create(self).BUILDALL(%attrinit);
+        nqp::create(self).BUILDALL(Empty, %attrinit);
     }
 
-    proto method BUILDALL(|) { * }
-
-    # This candidate provided for those modules that rely on the old
-    # BUILDALL interface, such as Inline::Perl5
-    multi method BUILDALL(@positional,%attrinit) {
-        self.BUILDALL(%attrinit)
-    }
-
-    multi method BUILDALL(%attrinit) {
+    method BUILDALL(Mu:D: @autovivs, %attrinit) {
         my $init := nqp::getattr(%attrinit,Map,'$!storage');
         # Get the build plan. Note that we do this "low level" to
         # avoid the NQP type getting mapped to a Rakudo one, which
         # would get expensive.
-        my $build_plan :=
-          nqp::findmethod(self.HOW,'BUILDALLPLAN')(self.HOW, self);
-        my int $count = nqp::elems($build_plan);
-        my int $i     = -1;
-        my $task;
-        my $build;
-        my int $code;
-        my int $int;
-        my num $num;
-        my str $str;
+        my $bp := nqp::findmethod(self.HOW,'BUILDALLPLAN')(self.HOW, self);
+        my int $count = nqp::elems($bp);
+        my int $i = -1;
 
         nqp::while(
           nqp::islt_i($i = nqp::add_i($i,1),$count),
 
-          nqp::if( # 0     # Custom BUILD call.
-            nqp::iseq_i(($code = nqp::atpos(
-              ($task := nqp::atpos($build_plan,$i)),0
-            )),0),
-            nqp::if(
+          nqp::if(
+            nqp::istype((my $task := nqp::atpos($bp,$i)),Callable),
+            nqp::if(                             # BUILD/TWEAK
               nqp::istype(
-                ($build := nqp::atpos($task,1)(self,|%attrinit)),Failure),
+                (my $build := nqp::if(
+                  nqp::elems($init),
+                  $task(self,|%attrinit),
+                  $task(self)
+                )),
+                Failure
+              ),
               return $build
             ),
 
-            nqp::if( # 1
-              nqp::iseq_i($code,1),
-              nqp::if(
-                nqp::existskey($init,nqp::atpos($task,2)),
-                (nqp::getattr(self,nqp::atpos($task,1),nqp::atpos($task,3))
-                  = %attrinit.AT-KEY(nqp::p6box_s(nqp::atpos($task,2))))
-              ),
+            nqp::if(                             # not just calling
+              (my int $code = nqp::atpos($task,0)),
 
-              nqp::if( # 2
-                nqp::iseq_i($code,2),
-                nqp::if(
-                  nqp::existskey($init,nqp::atpos($task,2)),
-                  (nqp::getattr(self,nqp::atpos($task,1),nqp::atpos($task,3))
-                    = %attrinit.AT-KEY(nqp::p6box_s(nqp::atpos($task,2)))),
-                  nqp::bindattr(self,nqp::atpos($task,1),nqp::atpos($task,3),
-                    nqp::list)
+              nqp::if(                           # >0
+                nqp::isle_i($code,3),
+                nqp::if(                         # 1|2|3
+                  nqp::existskey($init,nqp::atpos($task,3)),
+                  nqp::if(                       # can initialize
+                    nqp::iseq_i($code,1),
+                    nqp::bindattr_i(self,        # 1
+                      nqp::atpos($task,1),
+                      nqp::atpos($task,2),
+                      nqp::decont(%attrinit.AT-KEY(nqp::atpos($task,3)))
+                    ),
+                    nqp::if(
+                      nqp::iseq_i($code,2),
+                      nqp::bindattr_n(self,      # 2
+                        nqp::atpos($task,1),
+                        nqp::atpos($task,2),
+                        nqp::decont(%attrinit.AT-KEY(nqp::atpos($task,3)))
+                      ),
+                      nqp::bindattr_s(self,      # 3
+                        nqp::atpos($task,1),
+                        nqp::atpos($task,2),
+                        nqp::decont(%attrinit.AT-KEY(nqp::atpos($task,3)))
+                      )
+                    )
+                  )
                 ),
 
-                nqp::if( # 3
-                  nqp::iseq_i($code,3),
-                  nqp::if(
-                    nqp::existskey($init,nqp::atpos($task,2)),
-                    (nqp::getattr(self,nqp::atpos($task,1),nqp::atpos($task,3))
-                       = %attrinit.AT-KEY(nqp::p6box_s(nqp::atpos($task,2)))),
-                    nqp::bindattr(self,nqp::atpos($task,1),nqp::atpos($task,3),
-                       nqp::hash)
-                  ),
-
-                  nqp::if( # 4
-                    nqp::iseq_i($code,4),
-                    nqp::unless(
-                      nqp::attrinited(self,
-                        nqp::atpos($task,1),
-                        nqp::atpos($task,2)
-                      ),
+                nqp::if(
+                  nqp::iseq_i($code,4),
+                  nqp::unless(                   # 4
+                    nqp::attrinited(self,
+                      nqp::atpos($task,1),
+                      nqp::atpos($task,2)
+                    ),
+                    nqp::if(
+                      nqp::istype(nqp::atpos($task,3),Block),
                       nqp::stmts(
                         (my \attr := nqp::getattr(self,
                           nqp::atpos($task,1),
                           nqp::atpos($task,2)
                         )),
                         (attr = nqp::atpos($task,3)(self,attr))
-                      )
-                    ),
-
-                    nqp::if( # 5
-                      nqp::iseq_i($code,5),
-                      nqp::if(
-                        nqp::existskey($init,nqp::atpos($task,2)),
-                        nqp::bindattr_i(self,
-                          nqp::atpos($task,1),
-                          nqp::atpos($task,3),
-                          nqp::decont(%attrinit.AT-KEY(
-                            nqp::p6box_s(nqp::atpos($task,2))
-                          ))
-                        )
                       ),
-
-                      nqp::if( # 6
-                        nqp::iseq_i($code,6),
-                        nqp::if(
-                          nqp::existskey($init,nqp::atpos($task,2)),
-                          nqp::bindattr_n(self,
-                            nqp::atpos($task,1),
-                            nqp::atpos($task,3),
-                            nqp::decont(%attrinit.AT-KEY(
-                              nqp::p6box_s(nqp::atpos($task,2))
-                            ))
-                          )
-                        ),
-
-                        nqp::if( # 7
-                          nqp::iseq_i($code,7),
-                          nqp::if(
-                            nqp::existskey($init,nqp::atpos($task,2)),
-                            nqp::bindattr_s(self,
-                              nqp::atpos($task,1),
-                              nqp::atpos($task,3),
-                              nqp::decont(%attrinit.AT-KEY(
-                                nqp::p6box_s(nqp::atpos($task,2))
-                              ))
-                            )
-                          ),
-
-                          nqp::if( # 8
-                            nqp::iseq_i($code,8),
-                            nqp::if(
-                              nqp::iseq_i($int = nqp::getattr_i(self,
-                                nqp::atpos($task,1),
-                                nqp::atpos($task,2)
-                              ), 0),
-                              nqp::bindattr_i(self,
-                                nqp::atpos($task,1),
-                                nqp::atpos($task,2),
-                                (nqp::atpos($task,3)(self,$int))
-                              )
-                            ),
-
-                            nqp::if( # 9
-                              nqp::iseq_i($code,9),
-                              nqp::if(
-                                nqp::iseq_n($num = nqp::getattr_n(self,
-                                  nqp::atpos($task,1),
-                                  nqp::atpos($task,2)
-                                ), 0e0),
-                                nqp::bindattr_n(self,
-                                  nqp::atpos($task,1),
-                                  nqp::atpos($task,2),
-                                  (nqp::atpos($task,3)(self,$num))
-                                )
-                              ),
-
-                              nqp::if( # 10
-                                nqp::iseq_i($code,10),
-                                nqp::if(
-                                  nqp::isnull_s($str = nqp::getattr_s(self,
-                                    nqp::atpos($task,1),
-                                    nqp::atpos($task,2)
-                                  )),
-                                  nqp::bindattr_s(self,
-                                    nqp::atpos($task,1),
-                                    nqp::atpos($task,2),
-                                    (nqp::atpos($task,3)(self,$str))
-                                  )
-                                ),
-
-                              nqp::if( # 11
-                                nqp::iseq_i($code,11),
-                                nqp::unless(
-                                  nqp::attrinited(self,
-                                    nqp::atpos($task,1),
-                                    nqp::atpos($task,2)
-                                  ),
-                                  X::Attribute::Required.new(
-                                    name => nqp::atpos($task,2),
-                                    why  => nqp::atpos($task,3)
-                                  ).throw
-                                ),
-
-                                nqp::if( # 12
-                                  nqp::iseq_i($code,12),
-                                  nqp::bindattr(self,
-                                    nqp::atpos($task,1),
-                                    nqp::atpos($task,2),
-                                    (nqp::atpos($task,3)())
-                                  ),
-
-                                  nqp::if( # 13
-                                    nqp::isne_i($code,13),  # no-op
-                                    die("Invalid BUILDALL plan")
-                                  )
-        ))))))))))))));
-        self
-    }
-
-    method BUILD_LEAST_DERIVED(%attrinit) {
-        my $init := nqp::getattr(%attrinit,Map,'$!storage');
-        # Get the build plan for just this class.
-        my $build_plan := nqp::findmethod(self.HOW,'BUILDPLAN')(self.HOW,self);
-        my int $count = nqp::elems($build_plan);
-        my int $i     = -1;
-        my $task;
-        my $build;
-        my int $code;
-        my int $int;
-        my num $num;
-        my str $str;
-
-        nqp::while(
-          nqp::islt_i($i = nqp::add_i($i,1),$count),
-
-          nqp::if( # 0     # Custom BUILD call.
-            nqp::iseq_i(($code = nqp::atpos(
-              ($task := nqp::atpos($build_plan,$i)),0
-            )),0),
-            nqp::if(
-              nqp::istype(
-                ($build := nqp::atpos($task,1)(self,|%attrinit)),Failure),
-              return $build
-            ),
-
-            nqp::if( # 1
-              nqp::iseq_i($code,1),
-              nqp::if(
-                nqp::existskey($init,nqp::atpos($task,2)),
-                (nqp::getattr(self,nqp::atpos($task,1),nqp::atpos($task,3))
-                  = nqp::decont(
-                    %attrinit.AT-KEY(nqp::p6box_s(nqp::atpos($task,2)))
-                  )
-                )
-              ),
-
-              nqp::if( # 2
-                nqp::iseq_i($code,2),
-                nqp::if(
-                  nqp::existskey($init,nqp::atpos($task,2)),
-                  (nqp::getattr(self,nqp::atpos($task,1),nqp::atpos($task,3))
-                    = nqp::decont(
-                      %attrinit.AT-KEY(nqp::p6box_s(nqp::atpos($task,2)))
+                      nqp::getattr(self,nqp::atpos($task,1),nqp::atpos($task,2)) =
+                        nqp::atpos($task,3)
                     )
                   ),
-                  nqp::bindattr(self,nqp::atpos($task,1),nqp::atpos($task,3),
-                    nqp::list)
-                ),
 
-                nqp::if( # 3
-                  nqp::iseq_i($code,3),
                   nqp::if(
-                    nqp::existskey($init,nqp::atpos($task,2)),
-                    (nqp::getattr(self,nqp::atpos($task,1),nqp::atpos($task,3))
-                       = nqp::decont(
-                         %attrinit.AT-KEY(nqp::p6box_s(nqp::atpos($task,2)))
-                       )
-                    ),
-                    nqp::bindattr(self,nqp::atpos($task,1),nqp::atpos($task,3),
-                       nqp::hash)
-                  ),
-
-                  nqp::if( # 4
-                    nqp::iseq_i($code,4),
-                    nqp::unless(
-                      nqp::attrinited(self,
+                    nqp::iseq_i($code,5),
+                    nqp::if(                     # 5
+                      nqp::iseq_i(my $int = nqp::getattr_i(self,
                         nqp::atpos($task,1),
                         nqp::atpos($task,2)
+                      ), 0),
+                      nqp::bindattr_i(self,
+                        nqp::atpos($task,1),
+                        nqp::atpos($task,2),
+                        nqp::if(
+                          nqp::istype(nqp::atpos($task,3),Block),
+                          (nqp::atpos($task,3)(self,$int)),
+                          nqp::atpos($task,3)
+                        )
+                      )
+                    ),
+
+                    nqp::if(
+                      nqp::iseq_i($code,6),
+                      nqp::if(                   # 6
+                        nqp::iseq_n(my num $num = nqp::getattr_n(self,
+                          nqp::atpos($task,1),
+                          nqp::atpos($task,2)
+                        ), 0e0),
+                        nqp::bindattr_n(self,
+                          nqp::atpos($task,1),
+                          nqp::atpos($task,2),
+                          nqp::if(
+                            nqp::istype(nqp::atpos($task,3),Block),
+                            (nqp::atpos($task,3)(self,$num)),
+                            nqp::atpos($task,3)
+                          )
+                        )
                       ),
+
+                      nqp::if(
+                        nqp::iseq_i($code,7),
+                        nqp::if(                 # 7
+                          nqp::isnull_s(my str $str = nqp::getattr_s(self,
+                            nqp::atpos($task,1),
+                            nqp::atpos($task,2)
+                          )),
+                          nqp::bindattr_s(self,
+                            nqp::atpos($task,1),
+                            nqp::atpos($task,2),
+                            nqp::if(
+                              nqp::istype(nqp::atpos($task,3),Block),
+                              (nqp::atpos($task,3)(self,$str)),
+                              nqp::atpos($task,3)
+                            )
+                          )
+                        ),
+
+                      nqp::if(
+                        nqp::iseq_i($code,8),
+                        nqp::unless(             # 8
+                          nqp::attrinited(self,
+                            nqp::atpos($task,1),
+                            nqp::atpos($task,2)
+                          ),
+                          X::Attribute::Required.new(
+                            name => nqp::atpos($task,2),
+                            why  => nqp::atpos($task,3)
+                          ).throw
+                        ),
+
+                        nqp::if(
+                          nqp::iseq_i($code,9),
+                          nqp::bindattr(self,    # 9
+                            nqp::atpos($task,1),
+                            nqp::atpos($task,2),
+                            (nqp::atpos($task,3)())
+                          ),
+                          nqp::if(
+                            nqp::iseq_i($code,11),
+                            nqp::if(             # 11
+                              nqp::existskey($init,nqp::atpos($task,3)),
+                              (nqp::getattr(self,
+                                nqp::atpos($task,1),nqp::atpos($task,2))
+                                = %attrinit.AT-KEY(nqp::atpos($task,3))),
+                              nqp::bindattr(self,
+                                nqp::atpos($task,1),nqp::atpos($task,2),
+                                nqp::list
+                              )
+                            ),
+                            nqp::if(
+                              nqp::iseq_i($code,12),
+                              nqp::if(           # 12
+                                nqp::existskey($init,nqp::atpos($task,3)),
+                                (nqp::getattr(self,
+                                  nqp::atpos($task,1),nqp::atpos($task,2))
+                                  = %attrinit.AT-KEY(nqp::atpos($task,3))),
+                                nqp::bindattr(self,
+                                  nqp::atpos($task,1),nqp::atpos($task,2),
+                                  nqp::hash
+                                )
+                              ),
+                              die('Invalid ' ~ self.^name ~ ".BUILDALL plan: $code"),
+                  ))))))))),
+
+                  nqp::if(                       # 0
+                    nqp::existskey($init,nqp::atpos($task,3)),
+                    (nqp::getattr(self,nqp::atpos($task,1),nqp::atpos($task,2))
+                      = %attrinit.AT-KEY(nqp::atpos($task,3))),
+                  )
+                )
+              )
+            );
+            self
+        }
+
+        method BUILD_LEAST_DERIVED(%attrinit) {
+            my $init := nqp::getattr(%attrinit,Map,'$!storage');
+            # Get the build plan for just this class.
+            my $bp := nqp::findmethod(self.HOW,'BUILDPLAN')(self.HOW,self);
+            my int $count = nqp::elems($bp);
+            my int $i     = -1;
+
+        nqp::while(
+          nqp::islt_i($i = nqp::add_i($i,1),$count),
+
+          nqp::if(
+            nqp::istype((my $task := nqp::atpos($bp,$i)),Callable),
+            nqp::if(                             # BUILD/TWEAK
+              nqp::istype(
+                (my $build := nqp::if(
+                  nqp::elems($init),
+                  $task(self,|%attrinit),
+                  $task(self)
+                )),
+                Failure
+              ),
+              return $build
+            ),
+
+            nqp::if(                             # not just calling
+              (my int $code = nqp::atpos($task,0)),
+
+              nqp::if(                           # >0
+                nqp::isle_i($code,3),
+                nqp::if(                         # 1|2|3
+                  nqp::existskey($init,nqp::atpos($task,3)),
+                  nqp::if(                       # can initialize
+                    nqp::iseq_i($code,1),
+                    nqp::bindattr_i(self,        # 1
+                      nqp::atpos($task,1),
+                      nqp::atpos($task,2),
+                      nqp::decont(%attrinit.AT-KEY(nqp::atpos($task,3)))
+                    ),
+                    nqp::if(
+                      nqp::iseq_i($code,2),
+                      nqp::bindattr_n(self,      # 2
+                        nqp::atpos($task,1),
+                        nqp::atpos($task,2),
+                        nqp::decont(%attrinit.AT-KEY(nqp::atpos($task,3)))
+                      ),
+                      nqp::bindattr_s(self,      # 3
+                        nqp::atpos($task,1),
+                        nqp::atpos($task,2),
+                        nqp::decont(%attrinit.AT-KEY(nqp::atpos($task,3)))
+                      )
+                    )
+                  )
+                ),
+
+                nqp::if(
+                  nqp::iseq_i($code,4),
+                  nqp::unless(                   # 4
+                    nqp::attrinited(self,
+                      nqp::atpos($task,1),
+                      nqp::atpos($task,2)
+                    ),
+                    nqp::if(
+                      nqp::istype(nqp::atpos($task,3),Block),
                       nqp::stmts(
                         (my \attr := nqp::getattr(self,
                           nqp::atpos($task,1),
                           nqp::atpos($task,2)
                         )),
                         (attr = nqp::atpos($task,3)(self,attr))
+                      ),
+                      nqp::getattr(self,nqp::atpos($task,1),nqp::atpos($task,2)) =
+                        nqp::atpos($task,3)
+                    )
+                  ),
+
+                  nqp::if(
+                    nqp::iseq_i($code,5),
+                    nqp::if(                     # 5
+                      nqp::iseq_i(my $int = nqp::getattr_i(self,
+                        nqp::atpos($task,1),
+                        nqp::atpos($task,2)
+                      ), 0),
+                      nqp::bindattr_i(self,
+                        nqp::atpos($task,1),
+                        nqp::atpos($task,2),
+                        nqp::if(
+                          nqp::istype(nqp::atpos($task,3),Block),
+                          (nqp::atpos($task,3)(self,$int)),
+                          nqp::atpos($task,3)
+                        )
                       )
                     ),
 
-                    nqp::if( # 5
-                      nqp::iseq_i($code,5),
-                      nqp::if(
-                        nqp::existskey($init,nqp::atpos($task,2)),
-                        nqp::bindattr_i(self,
+                    nqp::if(
+                      nqp::iseq_i($code,6),
+                      nqp::if(                   # 6
+                        nqp::iseq_n(my num $num = nqp::getattr_n(self,
                           nqp::atpos($task,1),
-                          nqp::atpos($task,3),
-                          nqp::decont(%attrinit.AT-KEY(
-                            nqp::p6box_s(nqp::atpos($task,2))
-                          ))
+                          nqp::atpos($task,2)
+                        ), 0e0),
+                        nqp::bindattr_n(self,
+                          nqp::atpos($task,1),
+                          nqp::atpos($task,2),
+                          nqp::if(
+                            nqp::istype(nqp::atpos($task,3),Block),
+                            (nqp::atpos($task,3)(self,$num)),
+                            nqp::atpos($task,3)
+                          )
                         )
                       ),
 
-                      nqp::if( # 6
-                        nqp::iseq_i($code,6),
-                        nqp::if(
-                          nqp::existskey($init,nqp::atpos($task,2)),
-                          nqp::bindattr_n(self,
+                      nqp::if(
+                        nqp::iseq_i($code,7),
+                        nqp::if(                 # 7
+                          nqp::isnull_s(my str $str = nqp::getattr_s(self,
                             nqp::atpos($task,1),
-                            nqp::atpos($task,3),
-                            nqp::decont(%attrinit.AT-KEY(
-                              nqp::p6box_s(nqp::atpos($task,2))
-                            ))
+                            nqp::atpos($task,2)
+                          )),
+                          nqp::bindattr_s(self,
+                            nqp::atpos($task,1),
+                            nqp::atpos($task,2),
+                            nqp::if(
+                              nqp::istype(nqp::atpos($task,3),Block),
+                              (nqp::atpos($task,3)(self,$str)),
+                              nqp::atpos($task,3)
+                            )
                           )
                         ),
 
-                        nqp::if( # 7
-                          nqp::iseq_i($code,7),
-                          nqp::if(
-                            nqp::existskey($init,nqp::atpos($task,2)),
-                            nqp::bindattr_s(self,
-                              nqp::atpos($task,1),
-                              nqp::atpos($task,3),
-                              nqp::decont(%attrinit.AT-KEY(
-                                nqp::p6box_s(nqp::atpos($task,2))
-                              ))
-                            )
+                      nqp::if(
+                        nqp::iseq_i($code,8),
+                        nqp::unless(             # 8
+                          nqp::attrinited(self,
+                            nqp::atpos($task,1),
+                            nqp::atpos($task,2)
                           ),
+                          X::Attribute::Required.new(
+                            name => nqp::atpos($task,2),
+                            why  => nqp::atpos($task,3)
+                          ).throw
+                        ),
 
-                          nqp::if( # 8
-                            nqp::iseq_i($code,8),
-                            nqp::if(
-                              nqp::iseq_i($int = nqp::getattr_i(self,
+                        nqp::if(
+                          nqp::iseq_i($code,9),
+                          nqp::bindattr(self,    # 9
+                            nqp::atpos($task,1),
+                            nqp::atpos($task,2),
+                            (nqp::atpos($task,3)())
+                          ),
+                          nqp::if(
+                            nqp::iseq_i($code,10),
+                            # Force vivification, for the sake of meta-object
+                            # mix-ins at compile time ending up with correctly
+                            # shared containers.
+                            nqp::stmts(          # 10
+                              nqp::getattr(self,
                                 nqp::atpos($task,1),
                                 nqp::atpos($task,2)
-                              ), 0),
-                              nqp::bindattr_i(self,
-                                nqp::atpos($task,1),
-                                nqp::atpos($task,2),
-                                (nqp::atpos($task,3)(self,$int))
-                              )
-                            ),
-
-                            nqp::if( # 9
-                              nqp::iseq_i($code,9),
-                              nqp::if(
-                                nqp::iseq_n($num = nqp::getattr_n(self,
+                              ),
+                              nqp::while(        # 10's flock together
+                                nqp::islt_i(($i = nqp::add_i($i,1)),$count)
+                                  && nqp::iseq_i(
+                                       nqp::atpos(
+                                         ($task := nqp::atpos($bp,$i)),
+                                         0
+                                       ),10
+                                     ),
+                                nqp::getattr(self,
                                   nqp::atpos($task,1),
                                   nqp::atpos($task,2)
-                                ), 0e0),
-                                nqp::bindattr_n(self,
-                                  nqp::atpos($task,1),
-                                  nqp::atpos($task,2),
-                                  (nqp::atpos($task,3)(self,$num))
                                 )
                               ),
-
-                              nqp::if( # 10
-                                nqp::iseq_i($code,10),
-                                nqp::if(
-                                  nqp::isnull_s($str = nqp::getattr_s(self,
-                                    nqp::atpos($task,1),
-                                    nqp::atpos($task,2)
-                                  )),
-                                  nqp::bindattr_s(self,
-                                    nqp::atpos($task,1),
-                                    nqp::atpos($task,2),
-                                    (nqp::atpos($task,3)(self,$str))
+                              ($i = nqp::sub_i($i,1))
+                            ),
+                            nqp::if(
+                              nqp::iseq_i($code,11),
+                              nqp::if(           # 11
+                                nqp::existskey($init,nqp::atpos($task,3)),
+                                (nqp::getattr(self,
+                                  nqp::atpos($task,1),nqp::atpos($task,2))
+                                  = %attrinit.AT-KEY(nqp::atpos($task,3))),
+                                nqp::bindattr(self,
+                                  nqp::atpos($task,1),nqp::atpos($task,2),
+                                  nqp::list
+                                )
+                              ),
+                              nqp::if(
+                                nqp::iseq_i($code,12),
+                                nqp::if(         # 12
+                                  nqp::existskey($init,nqp::atpos($task,3)),
+                                  (nqp::getattr(self,
+                                    nqp::atpos($task,1),nqp::atpos($task,2))
+                                    = %attrinit.AT-KEY(nqp::atpos($task,3))),
+                                  nqp::bindattr(self,
+                                    nqp::atpos($task,1),nqp::atpos($task,2),
+                                    nqp::hash
                                   )
                                 ),
+                                die('Invalid ' ~ self.^name ~ ".BUILD_LEAST_DERIVED plan: $code"),
+              )))))))))),
 
-                                  nqp::if( # 13
-                                    nqp::iseq_i($code,13),
-                                    # Force vivification, for the sake of meta-object
-                                    # mix-ins at compile time ending up with correctly
-                                    # shared containers.
-                                    nqp::getattr(self,
-                                      nqp::atpos($task,1),
-                                      nqp::atpos($task,2)
-                                    ),
-                                    die("Invalid BUILD_LEAST_DERIVED plan")
-                                  )
-        ))))))))))));
+              nqp::if(                           # 0
+                nqp::existskey($init,nqp::atpos($task,3)),
+                (nqp::getattr(self,nqp::atpos($task,1),nqp::atpos($task,2))
+                  = %attrinit.AT-KEY(nqp::atpos($task,3))),
+              )
+            )
+          )
+        );
         self
     }
 
@@ -937,19 +977,30 @@ multi sub infix:<eqv>(Any:D \a, Any:D \b) {
 multi sub infix:<eqv>(Iterable:D \a, Iterable:D \b) {
     nqp::p6bool(
       nqp::unless(
-        nqp::eqaddr(a,b),                                    # identity
-        nqp::if(
-          nqp::eqaddr(a.WHAT,b.WHAT),                        # same type
-          nqp::if(
-            nqp::iseq_i((my int $elems = a.elems),b.elems),  # same # elems
-            nqp::stmts(
-              (my int $i = -1),
-              nqp::while(
-                nqp::islt_i(($i = nqp::add_i($i,1)),$elems)    # not exhausted
-                  && a.AT-POS($i) eqv b.AT-POS($i),          # still same
-                nqp::null
-              ),
-              nqp::iseq_i($i,$elems)                      # exhausted = success!
+        nqp::eqaddr(nqp::decont(a),nqp::decont(b)),
+        nqp::if(                                 # not same object
+          nqp::eqaddr(a.WHAT,b.WHAT),
+          nqp::if(                               # same type
+            a.is-lazy,
+            nqp::if(                             # a lazy
+              b.is-lazy,
+              die(X::Cannot::Lazy.new: :action<eqv>) # a && b lazy
+            ),
+            nqp::if(                             # a NOT lazy
+              b.is-lazy,
+              0,                                 # b lazy
+              nqp::if(                           # a && b NOT lazy
+                nqp::iseq_i((my int $elems = a.elems),b.elems),
+                nqp::stmts(                      # same # elems
+                  (my int $i = -1),
+                  nqp::while(
+                    nqp::islt_i(($i = nqp::add_i($i,1)),$elems) # not exhausted
+                      && a.AT-POS($i) eqv b.AT-POS($i),         # still same
+                    nqp::null
+                  ),
+                  nqp::iseq_i($i,$elems)         # exhausted = success!
+                )
+              )
             )
           )
         )
