@@ -7,12 +7,16 @@ my class ObjAt { # declared in BOOTSTRAP
 
     multi method WHICH(ObjAt:D:) {
         nqp::box_s(
-            nqp::concat(
-                nqp::concat(nqp::unbox_s(self.^name), '|'),
-                $!value
+          nqp::concat(
+            nqp::if(
+              nqp::eqaddr(self.WHAT,ObjAt),
+              'ObjAt|',
+              nqp::concat(nqp::unbox_s(self.^name), '|')
             ),
-            ObjAt
-        );
+            $!value
+          ),
+          ObjAt
+        )
     }
     multi method Str(ObjAt:D:) {
         nqp::p6box_s(nqp::unbox_s(self));
