@@ -67,6 +67,18 @@ my class Promise does Awaitable {
         $vow
     }
 
+    proto method kept(|) { * }
+    multi method kept(Promise:U:) {
+        my \rv := self.new;
+        rv!keep(True);
+        rv;
+    }
+    multi method kept(Promise:U: Mu \result) {
+        my \rv := self.new;
+        rv!keep(result);
+        rv;
+    }
+
     proto method keep(|) { * }
     multi method keep(Promise:D:) {
         self.vow.keep(True)
@@ -84,9 +96,21 @@ my class Promise does Awaitable {
         });
     }
 
+    proto method broken(|) { * }
+    multi method broken(Promise:U:) {
+        my \rv := self.new;
+        rv!break("Died");
+        rv;
+    }
+    multi method broken(Promise:U: Mu \exception) {
+        my \rv := self.new;
+        rv!break(exception);
+        rv;
+    }
+
     proto method break(|) { * }
     multi method break(Promise:D:) {
-        self.vow.break(False)
+        self.vow.break("Died")
     }
     multi method break(Promise:D: \result) {
         self.vow.break(result)
