@@ -1,7 +1,7 @@
 my class DateTime { ... }
 my role  IO { ... }
+my class IO::Handle { ... }
 my class IO::Path { ... }
-my class Seq { ... }
 my class Rakudo::Metaops { ... }
 my class X::Cannot::Lazy { ... }
 my class X::IllegalOnFixedDimensionArray { ... };
@@ -17,6 +17,20 @@ my class Rakudo::Internals {
     # for use in nqp::splice
     my $empty := nqp::list;
 
+    our class CompilerServices {
+        has Mu $!compiler;
+        has Mu $!current-match;
+
+        method generate_accessor(str $name, Mu \package_type, str $attr_name, Mu \type, int $rw) {
+            $!compiler.generate_accessor(
+              $!current-match, $name, package_type, $attr_name, type, $rw);
+        }
+        method generate_buildplan_executor(Mu \obj, Mu \buildplan) {
+            $!compiler.generate_buildplan_executor(
+              $!current-match, obj, buildplan)
+        }
+    }
+
     # rotate nqp list to another given list without using push/pop
     method RotateListToList(\from,\n,\to) {
         nqp::stmts(
@@ -129,12 +143,6 @@ my class Rakudo::Internals {
         0;
     }
 
-    method THE_END {
-        my @END := nqp::p6bindattrinvres(nqp::create(List), List, '$!reified',
-            nqp::getcurhllsym("@END_PHASERS"));
-        for @END -> $end { $end() };
-    }
-
     method createENV(int $bind) {
         nqp::stmts(
           (my $hash := nqp::hash),
@@ -298,10 +306,10 @@ my class Rakudo::Internals {
         )
     }
 
-    method TRANSPOSE(str $string, str $original, str $final) {
+    method TRANSPOSE(Str:D $string, Str:D $original, Str:D $final) {
         nqp::join($final,nqp::split($original,$string))
     }
-    method TRANSPOSE-ONE(str $string, str $original, str $final) {
+    method TRANSPOSE-ONE(Str:D $string, Str:D $original, Str:D $final) {
         nqp::if(
           nqp::iseq_i((my int $index = nqp::index($string, $original)), -1),
           $string,
@@ -506,7 +514,7 @@ my class Rakudo::Internals {
             $!buffer-start-seq = 0;
             $!done-target = -1;
             $!bust = 0;
-            $!lock := Lock.new;
+            $!lock := Lock::Async.new;
         }
 
         method process(Mu \seq, Mu \data, Mu \err) {
@@ -629,6 +637,9 @@ my class Rakudo::Internals {
     my num $init-time-num = nqp::time_n;
     method INITTIME() { $init-time-num }
 
+    my $init-thread := nqp::currentthread();
+    method INITTHREAD() { $init-thread }
+
     my $escapes := nqp::hash(
      "\0",   '\0',
      '$',    '\$',
@@ -878,9 +889,7 @@ my class Rakudo::Internals {
 #method INITIALIZERS() { $initializers }
 
     method REGISTER-DYNAMIC(Str:D \name, &code, Str $version = '6.c' --> Nil) {
-#nqp::print("Registering ");
-#nqp::print(name);
-#nqp::print("\n");
+#nqp::say('Registering ' ~ name);
         nqp::stmts(
           (my str $with = nqp::concat($version, nqp::concat("\0", name))),
           nqp::if(
@@ -898,9 +907,7 @@ my class Rakudo::Internals {
         )
     }
     method INITIALIZE-DYNAMIC(str \name) is raw {
-#nqp::print("Initializing");
-#nqp::print(name);
-#nqp::print("\n");
+#nqp::say('Initializing ' ~ name);
         nqp::stmts(
           (my str $with = nqp::concat(
             nqp::getcomp('perl6').language_version, nqp::concat("\0", name))),
@@ -1253,14 +1260,6 @@ my class Rakudo::Internals {
         nqp::stat_time(nqp::unbox_s(abspath), nqp::const::STAT_CHANGETIME)
     }
 
-    our class CompilerServices {
-        has Mu $!compiler;
-
-        method generate_accessor(str $name, Mu \package_type, str $attr_name, Mu \type, int $rw) {
-            $!compiler.generate_accessor($name, package_type, $attr_name, type, $rw);
-        }
-    }
-
     method HANDLE-NQP-SPRINTF-ERRORS(Mu \exception) {
         my $vmex := nqp::getattr(nqp::decont(exception), Exception, '$!ex');
         my \payload := nqp::getpayload($vmex);
@@ -1528,20 +1527,46 @@ my class Rakudo::Internals {
 # expose the number of bits a native int has
 my constant $?BITS = nqp::isgt_i(nqp::add_i(2147483648, 1), 0) ?? 64 !! 32;
 
+{   # setting up END phaser handling
+    my int $the-end-is-done;
+    my $the-end-locker = Lock.new;
+    # END handling, returns trueish if END handling already done/in progress
+    nqp::bindcurhllsym('&THE_END', {
+        unless $the-end-is-done {
+            $the-end-locker.protect: {
+                unless $the-end-is-done {
+                    my $comp := nqp::getcomp('perl6');
+                    my $end  := nqp::getcurhllsym('@END_PHASERS');
+                    while nqp::elems($end) {           # run all END blocks
+                        my $result := nqp::shift($end)();
+                        $result.sink if nqp::can($result,'sink');
+                        CATCH { $comp.handle-exception($_) }
+                        CONTROL { $comp.handle-control($_) }
+                    }
+#?if moar
+                    # close all open files
+                    IO::Handle.^find_private_method(
+                      'close-all-open-handles'
+                    )(IO::Handle);
+#?endif
+                    nqp::not_i(($the-end-is-done = 1)); # we're really done now
+                }
+            }
+        }
+    } );
+}
+
 # we need this to run *after* the mainline of Rakudo::Internals has run
 Rakudo::Internals.REGISTER-DYNAMIC: '&*EXIT', {
     PROCESS::<&EXIT> := sub exit($status) {
-        state $exit;
-        $exit = $status;
+        state $exit = $status;  # first call to exit sets value
 
-        once {
-            Rakudo::Internals.THE_END();
-            nqp::exit(nqp::unbox_i($exit.Int));
-        }
-        $exit;
+        nqp::getcurhllsym('&THE_END')()
+          ?? $exit
+          !! nqp::exit(nqp::unbox_i($exit.Int))
     }
 }
 
-sub exit($status = 0) { &*EXIT($status) }
+sub exit(Int(Any) $status = 0) { &*EXIT($status) }
 
 # vim: ft=perl6 expandtab sw=4
