@@ -89,7 +89,7 @@ my role Real does Numeric {
                 :range<0..1073741824>
             ) if $digits.defined and $digits < 0;
         my $prec = $digits // 1e8.log($base.Num).Int;
-        my Int $int_part = self.Int;
+        my Int $int_part = self.Int.self; # .self blows up Failures
         my $frac = abs(self - $int_part);
         my @frac_digits;
         my @conversion := <0 1 2 3 4 5 6 7 8 9
@@ -149,10 +149,12 @@ multi sub infix:<==>(Real \a, Real \b)  { a.Bridge == b.Bridge }
 multi sub infix:«<»(Real \a, Real \b)   { a.Bridge < b.Bridge }
 
 multi sub infix:«<=»(Real \a, Real \b)  { a.Bridge <= b.Bridge }
+multi sub infix:«≤» (Real \a, Real \b)  { a.Bridge  ≤ b.Bridge }
 
 multi sub infix:«>»(Real \a, Real \b)   { a.Bridge > b.Bridge }
 
 multi sub infix:«>=»(Real \a, Real \b)  { a.Bridge >= b.Bridge }
+multi sub infix:«≥» (Real \a, Real \b)  { a.Bridge  ≥ b.Bridge }
 
 multi sub prefix:<->(Real:D \a)            { -a.Bridge }
 
