@@ -1,7 +1,17 @@
 # Waits for a promise to be kept or a channel to be able to receive a value
-# and, once it can, unwraps or returns the result. This should be made more
-# efficient by using continuations to suspend any task running in the thread
-# pool that blocks; for now, this cheat gets the basic idea in place.
+# and, once it can, unwraps or returns the result. Under Perl 6.c, await will
+# really block the calling thread. In 6.d, if the thread is on the thread pool
+# then a continuation will be taken, and the thread is freed up.
+
+my role X::Await::Died {
+    has $.await-backtrace;
+    multi method gist(::?CLASS:D:) {
+        "An operation first awaited:\n" ~
+            ((try $!await-backtrace ~ "\n") // '<unknown location>') ~
+            "Died with the exception:\n" ~
+            callsame().indent(4)
+    }
+}
 
 proto sub await(|) { * }
 multi sub await() {
@@ -13,10 +23,34 @@ multi sub await(Any:U $x) {
 multi sub await(Any:D $x) {
     die "Must specify a Promise, Channel, or Supply to await on (got a $x.^name())";
 }
+multi sub await(Promise:D $p) {
+    CATCH {
+        unless nqp::istype($_, X::Await::Died) {
+            ($_ but X::Await::Died(Backtrace.new(5))).rethrow
+        }
+    }
+    my $*RAKUDO-AWAIT-BLOCKING := True;
+    $*AWAITER.await($p)
+}
+multi sub await(Channel:D $c) {
+    CATCH {
+        unless nqp::istype($_, X::Await::Died) {
+            ($_ but X::Await::Died(Backtrace.new(5))).rethrow
+        }
+    }
+    my $*RAKUDO-AWAIT-BLOCKING := True;
+    $*AWAITER.await($c)
+}
+multi sub await(Supply:D $s) {
+    CATCH {
+        unless nqp::istype($_, X::Await::Died) {
+            ($_ but X::Await::Died(Backtrace.new(5))).rethrow
+        }
+    }
+    my $*RAKUDO-AWAIT-BLOCKING := True;
+    $*AWAITER.await($s)
+}
 multi sub await(Iterable:D $i) { $i.eager.map({ await $_ }) }
-multi sub await(Promise:D $p)  { $p.result }
-multi sub await(Channel:D $c)  { $c.receive }
-multi sub await(Supply:D $s)   { $s.wait }
 multi sub await(*@awaitables)  { @awaitables.eager.map({await $_}) }
 
 sub awaiterator(@promises) {
@@ -47,6 +81,4 @@ sub awaiterator(@promises) {
     }.new(@promises))
 }
 
-sub cas (\val,&code) { val = code(val) } # naive implementation of cas
-
 # vim: ft=perl6 expandtab sw=4
